
import os
import shutil
import logging
import subprocess
from pathlib import Path
from typing import NamedTuple

from s3sel.utils import get_file_md5, query_yes_no


log = logging.getLogger(__file__)

class StoreException(Exception):
    pass

class ConfigFile(NamedTuple):
    name: str
    hash: str
    path: str


class ConfigStore:
    def __init__(self) -> None:
        self._cfg_files_path: Path = Path.home() / ".s3sel"
        self._s3cfg_path: Path = Path.home() / ".s3cfg"
        self._is_store_initialized = self._cfg_files_path.is_dir()
        self.cfg_list = self._read_all_config_files()
    
    def _read_all_config_files(self) -> list[ConfigFile]:
        if not self._is_store_initialized:
            return []
        
        all_files = []
        for p in self._cfg_files_path.glob("*"):
            filename = p.name
            cfg_name, cfg_hash = filename.split("-")
            all_files.append(
                ConfigFile(
                    hash=cfg_hash,
                    name=cfg_name,
                    path=p.as_posix(),
                )
            )

        return all_files 
    
    def _check_store_dir_or_create(self) -> None:
        if not self._is_store_initialized:
            log.info(".s3sel dir not found, creating...")
            self._cfg_files_path.mkdir()
    
    def _find_config_by_name(self, name: str) -> ConfigFile:
        cfg = next((c for c in self.cfg_list if c.name == name), None)
        if cfg is None:
            raise StoreException(f"No config with name \"{name}\" found")
        return cfg
    

    def add_new_config(self, config_name) -> str:
        assert self._s3cfg_path.is_file()

        s3cfg_md5 = get_file_md5(self._s3cfg_path)

        for file in self.cfg_list:
            if file.hash == s3cfg_md5:
                raise StoreException(
                    f"Such configuration file already exists with name: {file.name}"
                )
            
        new_path = self._cfg_files_path / f"{config_name}-{s3cfg_md5}"
        log.info("Saving config %s to %s", config_name, new_path.as_posix())
        self._check_store_dir_or_create()
        shutil.copy(self._s3cfg_path, new_path)
        return s3cfg_md5
    
    def get_config_by(self, key: str, value: str) -> ConfigFile | None:
        assert key in ConfigFile._fields
        for cfg in self.cfg_list:
            if getattr(cfg, key) == value:
                return cfg
        else:
            return None
    
    def remove_config_by_name(self, name: str):
        cfg = self._find_config_by_name(name)
        os.remove(cfg.path)
    
    def replace_current_config_by_name(self, name: str) -> ConfigFile | None:
        cfg = self._find_config_by_name(name)
        return self.replace_current_config(cfg)

    def replace_current_config(self, cfg: ConfigFile) -> ConfigFile | None:
        s3cfg_md5 = get_file_md5(self._s3cfg_path)
        if not self.get_config_by("hash", s3cfg_md5):
            if not query_yes_no(
                "You are going to overwrite .s3cfg which is not present in s3sel store. "
                "Are you sure?"
            ):
                raise StoreException("Rejected to overwrite config file.")
            
        shutil.copyfile(cfg.path, dst=self._s3cfg_path) 
        return cfg
    
    def get_current_config_hash(self) -> str:
        return get_file_md5(self._s3cfg_path)
        




import argparse
import logging
from pprint import pprint

from s3sel.files import ConfigStore, StoreException
from s3sel.ui import main_ui
from s3sel.utils import program_exit

log = logging.getLogger(__name__)


logging.basicConfig(level=logging.INFO)


def main_cli() -> None:
    parser = argparse.ArgumentParser(prog="s3sel") 
    parser.add_argument("-a", "--add-current", action="store_true", help="add your current .s3cfg to s3sel store")
    parser.add_argument("-l", "--list-all", action="store_true", help="show all configs present in s3sel store")
    parser.add_argument("-v", "--verbose", action="store_true", help="enable verbose output")
    parser.add_argument("-d", "--delete", action="store_true", help="delete config from s3sel store by specified name")
    parser.add_argument("-s", "--switch", action="store_true", help="switch your current .s3cfg to config with specified name")
    parser.add_argument("-c", "--check", action="store_true", help="check if your current .s3cfg is already stored")
    parser.add_argument("name", nargs="?")

    args = parser.parse_args()

    if not args.verbose:
        logging.disable(logging.INFO)

    store = ConfigStore()
    try:
        if args.add_current:
            if not args.name:
                program_exit(1, msg="No name specified for new config")
            name = args.name
            new_cfg_md5 = store.add_new_config(name)
            program_exit(0, msg=f"Added new config {name}, checksum: {new_cfg_md5}")
        elif args.delete:
            if not args.name:
                program_exit(1, msg="No name specified to delete")
            name = args.name
            store.remove_config_by_name(name)
            program_exit(0, msg=f"Deleted config with name \"{name}\"")
        elif args.list_all:
            pprint(store.cfg_list)
        elif args.switch:
            if not args.name:
                program_exit(1, msg="No name specified to switch to")
            name = args.name
            new_cfg = store.replace_current_config(name)
            program_exit(0, msg=f"Replaced current config with \"{new_cfg.name}\"")
        elif args.check:
            s3cfg_hash = store.get_current_config_hash()
            cfg = store.get_config_by("hash",  s3cfg_hash)
            if cfg:
                program_exit(0, msg=f"Config \"{cfg.name}\" with hash {s3cfg_hash} is already present in s3sel store.")
            else:
                program_exit(0, msg=f"No config with hash {s3cfg_hash} found.")
        else:
            main_ui(store)

    except StoreException as e:
        program_exit(1, msg=e.args[0])
    
    program_exit(0)

    

if __name__ == '__main__':
    main_cli()



from pprint import pformat

from simple_term_menu import TerminalMenu
from termcolor import colored

from s3sel.files import ConfigStore
from s3sel.utils import program_exit


def main_ui(store: ConfigStore) -> bool:
    cfg_map = {cfg.name: cfg for cfg in store.cfg_list}

    if not len(cfg_map):
        program_exit(0, msg="Can't run in ui mode, no configs stored. "
                     "(Try running with -h to see how to add new config.)")

    def get_preview(option_value: str) -> str:
        cfg = cfg_map.get(option_value)
        return pformat(cfg._asdict(), width=20)

    options = [cfg.name for cfg in cfg_map.values()]

    current_cfg = store.get_config_by("hash", store.get_current_config_hash())

    terminal_menu = TerminalMenu(
        menu_entries=options,
        preview_command=get_preview,
        preview_title=f"Current config: {colored(current_cfg.name, "cyan", attrs=["bold"])}"
    )
    menu_entry_index = terminal_menu.show()
    if menu_entry_index is None:
        return 
    selected_cfg = options[menu_entry_index]
    store.replace_current_config_by_name(selected_cfg)
    program_exit(0, msg=f"Replaced current config with \"{selected_cfg}\"")
    